unit BudgForm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Grids, StdCtrls, ExtCtrls, Buttons,
  BudgAcct;

type
  TMainForm = class(TForm)
    DrawGridAcct: TDrawGrid;
    EditPaycheckAmt: TEdit;
    ButtonPaycheck: TBitBtn;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    LabelBalance: TLabel;
    LabelPerCheck: TLabel;
    LabelPerMonth: TLabel;
    LabelPayCheck: TLabel;
    LabelPayMonth: TLabel;
    LabelRemainCheck: TLabel;
    LabelRemainMonth: TLabel;
    LabelCashBal: TLabel;
    LabelCCBal: TLabel;
    LabelAcctBal: TLabel;
    Panel1: TPanel;
    SButtonExit: TSpeedButton;
    SButtonPrint: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    PrintDialog: TPrintDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DrawGridAcctDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure DrawGridAcctDblClick(Sender: TObject);
    procedure ButtonPaycheckClick(Sender: TObject);
    procedure EditPaycheckAmtChange(Sender: TObject);
    procedure LabelBalanceDblClick(Sender: TObject);
    procedure LabelCashBalClick(Sender: TObject);
    procedure LabelCCBalClick(Sender: TObject);
    procedure DrawGridAcctKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SButtonExitClick(Sender: TObject);
    procedure SButtonPrintClick(Sender: TObject);
  private
    { Private declarations }
    accts : TAccounts;
    procedure UpdateNumbers;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

{==============================================================================}
implementation

{$R *.DFM}

uses
  BudgUpdt, BudgEdit, RptCol;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  accts := TAccounts.Create( Self );
  ReadComponentResFile( 'BUDGET.DAT', accts );
  EditPaycheckAmt.Text := FloatToStrF( accts.PaycheckAmt, ffFixed, 15, 2 );
  UpdateNumbers;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  WriteComponentResFile( 'BUDGET.DAT', accts );
  accts.Free;
end;

procedure TMainForm.SButtonExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.DrawGridAcctDrawCell(Sender: TObject; Col,
  Row: Longint; Rect: TRect; State: TGridDrawState);
var
  s    : string;
  fmt  : word;
  acct : TAcct;
begin
  s := '';
  if (Row = 0) then
  case Col of
    0 : s := 'Account Name';
    1 : s := 'Monthly';
    2 : s := 'Per Check';
    3 : s := 'Balance';
  end
  else
  begin
    if (accts.Count >= Row) then
    try
      acct := accts.Account[Row-1];
    except
      on EListError do acct := nil;
    end
    else
      acct := nil;

    if (acct <> nil) then
    case Col of
      0 : s := acct.Name;
      1 : s := FloatToStrF( acct.AmtPerMonth, ffCurrency, 15, 2 );
      2 : s := FloatToStrF( acct.AmtPerPaycheck, ffCurrency, 15, 2 );
      3 : s := FloatToStrF( acct.Balance, ffCurrency, 15, 2 );
    end;
  end;

  Inc( Rect.Left, 1 );
  Dec( Rect.Right, 1 );
  if (Col > 0) then
    fmt := DT_RIGHT
  else
    fmt := DT_LEFT;
  with Sender as TDrawGrid do
    DrawText( Canvas.Handle, @s[1], Ord(s[0]), Rect, fmt );
end;

procedure TMainForm.DrawGridAcctDblClick(Sender: TObject);
var
  acct     : TAcct;
  editForm : TDialogAcctUpdate;
begin
  with Sender as TDrawGrid do
  begin
    if (Row > 0) and (Row <= accts.Count) then
      acct := accts.Account[Row-1]
    else
    begin
      accts.AddAccount( 'New Account', 0.0, 0.0 );
      acct := accts.Account[accts.Count-1];
    end;
    editForm := TDialogAcctUpdate.Create( Application );
    editForm.Execute( acct, accts );
  end;
  UpdateNumbers;
end;

procedure TMainForm.ButtonPaycheckClick(Sender: TObject);
begin
  accts.DistributePaycheck;
  UpdateNumbers;
end;

procedure TMainForm.EditPaycheckAmtChange(Sender: TObject);
begin
  accts.PaycheckAmt := StrToFloat( (Sender as TEdit).Text );
  UpdateNumbers;
end;

procedure TMainForm.UpdateNumbers;
var
  bdgMonthly,
  payMonthly : double;
  bdgCheck,
  payCheck   : double;
  balance    : double;
  cash       : double;
  creditCard : double;
begin
  DrawGridAcct.Invalidate;
  bdgMonthly := accts.BudgetMonthly;
  payMonthly := Round( (accts.PaycheckAmt*26)/12 );
  bdgCheck := accts.BudgetPaycheck;
  payCheck := accts.PaycheckAmt;
  balance := accts.Balance;
  cash := accts.Cash;
  creditCard := accts.CreditCard;
  LabelPerMonth.Caption := FloatToStrF( bdgMonthly, ffCurrency, 15, 2 );
  LabelPayMonth.Caption := FloatToStrF( payMonthly, ffCurrency, 15, 2 );
  LabelRemainMonth.Caption := FloatToStrF( payMonthly-bdgMonthly, ffCurrency, 15, 2 );
  LabelPerCheck.Caption := FloatToStrF( bdgCheck, ffCurrency, 15, 2 );
  LabelPayCheck.Caption := FloatToStrF( payCheck, ffCurrency, 15, 2 );
  LabelRemainCheck.Caption := FloatToStrF( payCheck-bdgCheck, ffCurrency, 15, 2 );
  LabelBalance.Caption := FloatToStrF( balance, ffCurrency, 15, 2 );
  LabelCashBal.Caption := FloatToStrF( cash, ffCurrency, 15, 2 );
  LabelCCBal.Caption := FloatToStrF( creditCard, ffCurrency, 15, 2 );
  LabelAcctBal.Caption := FloatToStrF( balance+creditCard-cash, ffCurrency, 15, 2 );
end;

procedure TMainForm.LabelBalanceDblClick(Sender: TObject);
begin
  if (MessageDlg('Clear all balances?',mtConfirmation,[mbYes,mbNo],0) = mrYes) then
    accts.ClearBalances;
  UpdateNumbers;
end;

procedure TMainForm.LabelCashBalClick(Sender: TObject);
var
  amt      : double;
  editForm : TDialogEditAmount;
begin
  editForm := TDialogEditAmount.Create( Application );
  amt := accts.Cash;
  editForm.Execute( 'Cash', amt );
  accts.Cash := amt;
  UpdateNumbers;
end;

procedure TMainForm.LabelCCBalClick(Sender: TObject);
var
  amt      : double;
  editForm : TDialogEditAmount;
begin
  editForm := TDialogEditAmount.Create( Application );
  amt := accts.CreditCard;
  editForm.Execute( 'Credit Card', amt );
  accts.CreditCard := amt;
  UpdateNumbers;
end;

procedure TMainForm.DrawGridAcctKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_RETURN : DrawGridAcctDblClick( Sender );
    VK_INSERT :
      with Sender as TDrawGrid do
        if (Row > 0) and (Row <= accts.Count) then
        begin
          accts.InsertAccount( Row-1, 'New Account', 0.0, 0.0 );
          UpdateNumbers;
          DrawGridAcctDblClick( Sender );
        end;
    VK_DELETE :
      with Sender as TDrawGrid do
        if (Row > 0) and (Row <= accts.Count) and
           (MessageDlg('Delete account?',mtConfirmation,[mbYes,mbNo],0) = mrYes) then
        begin
          accts.DeleteAccount( Row-1 );
          UpdateNumbers;
        end;

  else
    FormKeyDown( Sender, Key, Shift );
  end;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssAlt in Shift) then
  case Key of
    Ord('C') : LabelCashBalClick( LabelCashBal );
    Ord('R') : LabelCCBalClick( LabelCCBal );
  end;
end;

procedure TMainForm.SButtonPrintClick(Sender: TObject);
var
  Report : TColumnReport;
  acct   : TAcct;
  cntr   : word;
begin
  if PrintDialog.Execute then
  begin
    Report := TColumnReport.Create;
    try
      Report.Font.Name := 'Courier New';
      Report.Font.Size := 20;
      Report.Start;
      Report.AddField( 'Account', '%s', 30, jfLeft );
      Report.AddField( 'Monthly', '%9.2n', 10, jfRight );
      Report.AddField( 'Per Check', '%9.2n', 10, jfRight );
      Report.AddField( 'Balance', '%9.2n', 10, jfRight );
      Report.PrintString(
        Format( 'BUDGET REPORT (Date: %s)', [FormatDateTime('dddddd',Now)] ) );
      Report.IncLine( 2 );

      for cntr := 0 to accts.Count - 1 do
      begin
        acct := accts.Account[cntr];
        with acct do
          Report.PrintLine( [Name, AmtPerMonth, AmtPerPaycheck, Balance] );
      end;

      Report.Stop;
    finally
      Report.Free;
    end;
  end;

end;

end.
